package SliMP3::History;

# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License, 
# version 2.

use strict;
use File::Spec::Functions qw(:ALL);
use POSIX;
use SliMP3::Misc;

use SliMP3::Strings qw(string);

# a cache of songs played
my @history = ();

# Clear can be used to clear the history cache.
sub clear {
	@history =();
};

# recount processes the history list by counting the number of times a song appears, and then
# sorting the list based on that number.  The sorted list is returned.  The returned list 
# will be a two dimensional array (N by 2).  Each element contains the song name, and the number
# of times it appeared in the history.
sub recount {
	my @outlist;
	my $listcount = 0;
	my $i=1;
	my $item;
	my $title;
	my $count;
	
	@outlist = ();
	
    if (scalar(@history)) {        
        
	    # Cycle through the history and count songs. 
        ITEM: foreach $item (@history) {
    		for ($i = 1; $i <= $listcount; $i++) {
    			if ($item eq $outlist[$i][1]) {
            		$outlist[$i][2]++;
            		next ITEM;
    			}
    		}
    		# Add a two element array (containing the song and song's count) to the end of the ouput array.
    		$listcount++;
    		$outlist[$listcount][1] = $item;
    		$outlist[$listcount][2] = 1;
        }

        # Sort array by second element of each subarray (song count).  If there is a tie, the winner
        # should be the most recently played song of those that tie because of how the initial list 
        # was ordered.
    	for ($i = 1; $i < (scalar(@outlist)-1); $i++) {
    		if (($outlist[$i][2] <=> $outlist[$i+1][2])==-1) {
    		    $title = $outlist[$i][1];
    		    $count = $outlist[$i][2];
    		    $outlist[$i][1] = $outlist[$i+1][1];
    		    $outlist[$i][2] = $outlist[$i+1][2];
    		    $outlist[$i+1][1] = $title;
        	    $outlist[$i+1][2] = $count;
        	    $i = 0;
    		}
    	}
    }

	return @outlist;
};

# Record takes a song name and stores it at the first position of an array.  The max 
# length on the array is arbitrarily set at 1000.  
sub record {
    my $song = shift;
    
    # Add the newest song to the font of the list, so that the most recent song is at the top.
    unshift @history,$song;
    
    # Max history is 1000.  
    if (scalar(@history)>1000) { pop @history; }

};

# Histlist fills variables for populating an html file. 
sub hitlist {
	my($client, %params) = @_;
	my $output = "";
	my @items = ();
	my %list_form;
	my $i;
	my $song;
	my $itemnumber=0;
	my $maxplayed=0;

	@items = recount();
	if (scalar(@items)) {
		for ($i = 1; $i < scalar(@items); $i++) {
		    if ($maxplayed==0) { $maxplayed = $items[$i][2]; }
    		%list_form = ();
            $song = $items[$i][1];

			$list_form{'title'} 	= SliMP3::Info::standardTitle(undef,$song);
			$list_form{'artist'} 	= SliMP3::Info::artist($song);
			$list_form{'album'} 	= SliMP3::Info::album($song);
			$list_form{'odd'}	  = ($itemnumber + 1) % 2;
			$list_form{'song_bar'} = hitlist_bar( $items[$i][2], $maxplayed );
			$itemnumber++;

			$list_form{'song_count'} = $items[$i][2];
			$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("hitlist_list.html", \%list_form);
		}
	}

	$params{'total_song_count'} = SliMP3::Info::songCount();
	$params{'genre_count'} = SliMP3::Info::genreCount();
	$params{'artist_count'} = SliMP3::Info::artistCount();
	$params{'album_count'} = SliMP3::Info::albumCount();

	$output .= &SliMP3::HTTP::filltemplatefile("hitlist.html", \%params);
}

sub hitlist_bar {
    my $curr = shift;
    my $max = shift;
    my $returnval="";
    
    my %list_form=();
    
    $list_form{'cell_full'} = (($curr*100)/$max) > 9;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 19;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 29;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 39;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 49;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 59;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 69;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 79;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = (($curr*100)/$max) > 89;
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    $list_form{'cell_full'} = ($curr == $max);
    $returnval .= &SliMP3::HTTP::filltemplatefile("hitlist_bar.html", \%list_form);
    
    return $returnval;
}



	
1;
	