package SliMP3::Olson;

# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License, 
# version 2.

use strict;
use File::Spec::Functions qw(:ALL);
use POSIX;
use SliMP3::Misc;

use SliMP3::Strings qw(string);

my($NEWLINE) = "\012";

sub olsonmain {
	my($client, %params) = @_;
	my $output = "";
	my @items = ();
	my @songlist = ();

	my $song = $params{'song'};
	my $artist = $params{'artist'};
	my $album = $params{'album'};
	my $genre = $params{'genre'};
	my $player = $params{'player'};
	my $descend;
	my %list_form;
	my $itemnumber = 0;
	my $lastAnchor = '';

	# warn the user if the scanning isn't complete.
	if (SliMP3::MusicFolderScan::stillScanning() || SliMP3::iTunes::stillScanning()) {
		$params{'warn'} = 1;
	}

	if (SliMP3::iTunes::useiTunesLibrary()) {
		$params{'itunes'} = 1;
	}

	if (defined($genre) && $genre eq '*' && 
	    defined($artist) && $artist eq '*') {
		$params{'browseby'} = string('BROWSE_BY_ALBUM');
	} elsif (defined($genre) && $genre eq '*') {
		$params{'browseby'} = string('BROWSE_BY_ARTIST');
	} else {
	    $params{'browseby'} = string('BROWSE_BY_GENRE');
	};

	if (defined($genre) && $genre ne '*' && 
	    defined($artist) && $artist ne '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'artist'} = '';
		$list_form{'album'} = '';
		$list_form{'genre'} = '';
		$list_form{'player'} = $player;
	        $list_form{'pwditem'} = string('BROWSE_BY_GENRE');
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("olsonmain_pwdlist.html", \%list_form);
	      $params{'browseby'} = $genre;
	};

	my $otherparams = 'player=' . SliMP3::HTTP::escape($player) . 
					  '&genre=' . SliMP3::HTTP::escape($genre) . 
					  '&artist=' . SliMP3::HTTP::escape($artist) . 
					  '&album=' . SliMP3::HTTP::escape($album) . 
					  '&song=' . SliMP3::HTTP::escape($song) . '&';
	if (!$genre) {
		@items = SliMP3::Info::genres([], [$artist], [$album], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = SliMP3::Pages::alphapagebar(\@items,$params{'path'},$otherparams,\$params{'start'},\$params{'browselist_pagebar'});
			$descend = 'true';
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $item;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = $album;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;

				@songlist = SliMP3::Info::songs([$item], [], [], []);
				$list_form{'song_count'} = addsongsuffix(scalar(@songlist));

				my $anchor = SliMP3::Pages::anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("olson_tomain.html", \%list_form);
			}
		}
	} elsif (!$artist) {
		@items = SliMP3::Info::artists([$genre], [], [$album], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = SliMP3::Pages::alphapagebar(\@items,$params{'path'},$otherparams,\$params{'start'},\$params{'browselist_pagebar'});
			$descend = 'true';
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $item;
				$list_form{'album'}	  = $album;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'} = ($itemnumber + 1) % 2;

				@songlist = SliMP3::Info::songs([], [$item], [], []);
				$list_form{'song_count'} = addsongsuffix(scalar(@songlist));

				my $anchor = SliMP3::Pages::anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("olson_todetail.html", \%list_form);
			}
		}
	} elsif (!$album) {
		@items = SliMP3::Info::albums([$genre], [$artist], [], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = SliMP3::Pages::alphapagebar(\@items,$params{'path'},$otherparams,\$params{'start'},\$params{'browselist_pagebar'});
			$descend = 'true';
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = $item;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;

				@songlist = SliMP3::Info::songs([$genre], [$artist], [$item], []);
				$list_form{'song_count'} = addsongsuffix(scalar(@songlist));

				my $anchor = SliMP3::Pages::anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("olson_todetail.html", \%list_form);
			}
		}
	}
	$params{'descend'} = $descend;

	$output .= &SliMP3::HTTP::filltemplatefile("olsonmain.html", \%params);
}


sub olsondetail {
	my($client, %params) = @_;
	my $output = "";
	my @items = ();
	my @songlist = ();

	my $song = $params{'song'};
	my $artist = $params{'artist'};
	my $album = $params{'album'};
	my $genre = $params{'genre'};
	my $player = $params{'player'};
	my $descend;
	my %list_form;
	my $itemnumber = 0;
	my $lastAnchor = '';

	# warn the user if the scanning isn't complete.
	if (SliMP3::MusicFolderScan::stillScanning() || SliMP3::iTunes::stillScanning()) {
		$params{'warn'} = 1;
	}

	if (SliMP3::iTunes::useiTunesLibrary()) {
		$params{'itunes'} = 1;
	}

	$params{'pwd_list'} = "";

	if (defined($artist) && $artist ne '' &&
	    $artist ne '*' && defined($album) && $album ne '') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'artist'} = $artist;
		$list_form{'album'} = '';
		$list_form{'genre'} = $genre;
		$list_form{'player'} = $player;
  	        $list_form{'pwditem'} = "Back to " . $artist;
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("olsondetail_pwdlist.html", \%list_form);
		$params{'browseby'} = $artist . ' / ' . $album;
	} elsif (defined($artist) && $artist ne '' && $artist ne '*') {
		$params{'browseby'} = $artist;
	} elsif (defined($album) && $album ne '') {
		$params{'browseby'} = $album;
	} 

	my $otherparams = 'player=' . SliMP3::HTTP::escape($player) . 
					  '&genre=' . SliMP3::HTTP::escape($genre) . 
					  '&artist=' . SliMP3::HTTP::escape($artist) . 
					  '&album=' . SliMP3::HTTP::escape($album) . 
					  '&song=' . SliMP3::HTTP::escape($song) . '&';
	if (!$album) {
		@items = SliMP3::Info::albums([$genre], [$artist], [], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = SliMP3::Pages::alphapagebar(\@items,$params{'path'},$otherparams,\$params{'start'},\$params{'browselist_pagebar'});
			$descend = 'true';
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = $item;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;

				@songlist = SliMP3::Info::songs([], [$artist], [$item], []);
				$list_form{'song_count'} = addsongsuffix(scalar(@songlist));

				my $anchor = SliMP3::Pages::anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("olson_todetail.html", \%list_form);
			}
		}
	} else {
		@items = SliMP3::Info::songs([$genre], [$artist], [$album], []);
		if (scalar(@items)) {
			my ($start,$end) = SliMP3::Pages::pagebar(scalar(@items),$params{'path'},0,$otherparams,\$params{'start'},\$params{'browselist_header'},\$params{'browselist_pagebar'});
			$descend = undef;
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				my $title = SliMP3::Info::standardTitle(undef, $item);
				$list_form{'genre'}	  = SliMP3::Info::genre($item);
				$list_form{'artist'}  = SliMP3::Info::artist($item);
				$list_form{'album'}	  = SliMP3::Info::album($item);
				$list_form{'itempath'} = SliMP3::Misc::safePath($item);
				$list_form{'title'}   = $title;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("olson_todetail.html", \%list_form);
			}
		}
	}
	$params{'descend'} = $descend;

	$output .= &SliMP3::HTTP::filltemplatefile("olsondetail.html", \%params);
}

sub addsongsuffix {
	my $number = shift;

	my $output = $number;
	if (scalar($number) > 1) { 
		$output .= ' songs'; 
	} else { 
		$output .= ' song'; 
	}
	return ($output);
}


1;